/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2021 Marcus Britanicus (https://gitlab.com/marcusbritanicus)
 * Copyright (c) 2021 Abrar (https://gitlab.com/s96Abrar)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 **/

#include <QDebug>
#include <QWindow>

#include <wayland-client.h>

#include <wayqt/WayQtUtils.hpp>
#include <wayqt/Registry.hpp>
#include <wayqt/SessionLock.hpp>

#include "ext-session-lock-v1-client-protocol.h"

#include "SessionLockImpl.hpp"


const struct ext_session_lock_surface_v1_listener SessionLockSurfaceImpl::mListener = {
    configureCallback,
};


SessionLockSurfaceImpl::SessionLockSurfaceImpl( QtWaylandClient::QWaylandWindow *window, ext_session_lock_surface_v1 *surf ) : QtWaylandClient::QWaylandShellSurface( window ) {
    mWindow  = window;
    mObj     = surf;

    ext_session_lock_surface_v1_add_listener( surf, &mListener, this );
    wl_display_roundtrip( WQt::Wayland::display() );
}


SessionLockSurfaceImpl::~SessionLockSurfaceImpl() {
    ext_session_lock_surface_v1_destroy( mObj );
}


ext_session_lock_surface_v1 *SessionLockSurfaceImpl::get() {
    return mObj;
}

void SessionLockSurfaceImpl::configureCallback( void *data, struct ::ext_session_lock_surface_v1 *, uint32_t serial, uint32_t width, uint32_t height ) {
    auto sls = reinterpret_cast<SessionLockSurfaceImpl *>(data);

    qCritical() << "Configure called";

    ext_session_lock_surface_v1_ack_configure( sls->mObj, serial );

    /** Apply the size request: This has to be obeyed. */
    sls->applyConfigure();
    #if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
        sls->window()->resizeFromApplyConfigure( {(int)width, (int)height} );

    #else
        sls->resizeFromApplyConfigure( {(int)width, (int)height} );

    #endif

    /** Commit the buffer */
    sls->handleExpose( QRect( QPoint(), QSize((int)width, (int)height ) ) );
}
